From 8442e8811d25863ee12538834edca0b5572bc54c Mon Sep 17 00:00:00 2001
From: Alyssa Rosenzweig <alyssa.rosenzweig@collabora.com>
Date: Wed, 27 Jan 2021 12:31:45 -0500
Subject: [PATCH] pan/bi: Implement ufind_msb

Lowered to #(sz - 1) - clz(x), taking advantage of the machine's
8-bit and 16-bit variants of clz and the widening on the second argument
of ISUB to implement neatly in two instructions.

Note that in NIR, ufind_msb can take any integer type but always output
i32.

Signed-off-by: Alyssa Rosenzweig <alyssa.rosenzweig@collabora.com>
---
 src/panfrost/bifrost/bifrost_compile.c | 6 ++++++
 1 file changed, 6 insertions(+)

diff --git a/src/panfrost/bifrost/bifrost_compile.c b/src/panfrost/bifrost/bifrost_compile.c
index 2084944306b..47df541c3ff 100644
--- a/src/panfrost/bifrost/bifrost_compile.c
+++ b/src/panfrost/bifrost/bifrost_compile.c
@@ -1325,6 +1325,12 @@ bi_emit_alu(bi_builder *b, nir_alu_instr *instr)
                 bi_bitrev_i32_to(b, dst, s0);
                 break;
 
+        case nir_op_ufind_msb: {
+                bi_index clz = bi_clz(b, src_sz, s0, false);
+                bi_isub_u32_to(b, dst, bi_imm_u32(src_sz - 1), clz, false);
+                break;
+        }
+
         default:
                 fprintf(stderr, "Unhandled ALU op %s\n", nir_op_infos[instr->op].name);
                 unreachable("Unknown ALU op");
-- 
2.27.0

